<?php
/*
Copyright (C) 2004-2007 The Polaroid-o-nizer Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

define("VERSION", "0.7.2");
define("CREATIONDATE", "August 13th, 2007");
define("TEMPDIR", "temp/");
header("PoN-Version: " . VERSION);
error_reporting(0);

require_once(dirname(__FILE__) . '/../../../wp-config.php');

if (get_potf_option('secure') == 1) {
	if ($_SERVER['QUERY_STRING'] == "") {
	        exit("This page cannot be called in this manner.");
	}

	$items = split('//',$_SERVER['HTTP_REFERER']);
	$site = split('/',$items[1]);
	if ($site[0] != $_ENV["HTTP_HOST"]) {
        	exit("You are trying to access this page from an illegal referrer ($site[0])");
	}
}

class Polaroid
{
	var $errmsg = array();
	var $photo;
	var $text;

	function Polaroid()
	{
		global $data;
		if (empty($data['bg']))
		{
			$this->errmsg['bg'] = "Missing background color value:";
		}
		else
		{
			$bg = explode(",", $data['bg']);
			for ($i = 0; $i < 3; $i++)
			{
				if ($bg[$i] < 0 || $bg[$i] > 255 || !is_numeric($bg[$i]))
				{
					$this->errmsg['bg'] = "Incorrect background color value:";
					break;
				}
			}
		}

		$data['x'] = !empty($data['x']) ? $data['x'] : 0;
		$data['y'] = !empty($data['y']) ? $data['y'] : 0;

		if ((isset($data['x']) && !is_numeric($data['x'])) || (isset($data['y']) && !is_numeric($data['y'])))
		{
			$this->errmsg['xy'] = "Incorrect x and/or y coordinates:";
		}

		$data['angle'] = !empty($data['angle']) || $data['angle'] == "0" ? $data['angle'] : 15;

		if ($data['angle'] < 0 || $data['angle'] > 360 || !is_numeric($data['angle']))
		{
			$this->errmsg['angle'] = "Incorrect rotation angle:";
		}

		if(!isset($data['photo']) || $data['photo'] == "http://")
		{
			$this->errmsg['photo'] = "Missing image URL:";
		}
		else
		{
			$this->photo = str_replace(" ", "%20", $data['photo']);
			$info = @getimagesize($this->photo);
			if(!$info)
			{
				$this->errmsg['photo'] = "Incorrect URL or linked file does not exist:";
			}
			elseif(!in_array($info[2], array(1, 2, 3)))
			{
				$this->errmsg['photo'] = "Unknown filetype. Use GIF, JPEG or PNG only:";
			}
			elseif (($info[0] - $data['x']) < 200 || ($info[1] - $data['y']) < 200)
			{
				$this->errmsg['photo'] = "Imageresolution is too low:";
			}
			elseif ($info[0] >= 2000 || $info[1] >= 2000)
			{
				$this->errmsg['photo'] = "Imageresolution is too high";
			}
		}

		$text = trim(strip_tags(stripslashes(str_replace("_", " ", $data['text']))));
		/*if(empty($text))
		{
			$this->errmsg['text'] = "Missing (funny) remark/text:";
		}
		else*/if(strlen($text) > 100)
		{
			$this->errmsg['text'] = "Text/remark is too long:";
		}

		$this->text = trim(strip_tags(stripslashes(str_replace(" ", "_", $data['text']))));
	}

	function CreatePolaroid($outputMethod)
	{
		global $data;
		$text = trim(strip_tags(stripslashes(str_replace("_", " ", $data['text']))));

		$polaroid = imagecreatetruecolor(246, 300);

		$bg = explode(",", $data['bg']);
		$bg = imagecolorallocate($polaroid, $bg[0], $bg[1], $bg[2]);
		imagefill($polaroid, 0, 0, $bg);

		$photo = str_replace(" ", "%20", $data['photo']);
		$info = getimagesize($photo);

		$scale = round(($info[0] > $info[1]) ? (200 / $info[1]) : (200 / $info[0]), 4);

		if ($info[2] == 1)
		{
			$photo = imagecreatefromgif(stripslashes($photo));
		}
		elseif ($info[2] == 2)
		{
			$photo = imagecreatefromjpeg(stripslashes($photo));
		}
		elseif ($info[2] == 3)
		{
			$photo = imagecreatefrompng(stripslashes($photo));
		}

		$tmp = imagecreatetruecolor(200, 200);
		imagecopyresampled($tmp, $photo, 0, 0, $data['x'], $data['y'], floor($info[0] * $scale), floor($info[1] * $scale), $info[0], $info[1]);

		imagecopy($polaroid, $tmp, 20, 18, 0, 0, 200, 200);

		$frame = imagecreatefrompng("frame.png");
		imagecopy($polaroid, $frame, 0, 0, 0, 0, 245, 301);

		$text = wordwrap($text, 25, "||", 1);
		$text = explode("||", $text);

		$black = imagecolorallocate($polaroid, 0, 0, 0);

		$text_pos_y = array(235, 250, 265);
		for ($i = 0; $i < 3; $i++)
		{
			$width = imagettfbbox(10, 0, "annifont.ttf", $text[$i]);
			$text_pos_x = (196 - $width[2])/2;
			imagettftext($polaroid, 18, 0, $text_pos_x, $text_pos_y[$i]+10, $black, "annifont.ttf", $text[$i]);
		}

		$polaroid = imagerotate($polaroid, $data['angle'], $bg);

		switch ($outputMethod)
		{
			case "upload":
				/**************************************************
				<Upload to ImageShack>
				**************************************************/
				$filename = rand(10000,99999).".jpg";
				imagejpeg($polaroid, TEMPDIR . $filename, 80);
				imagedestroy($polaroid);

				$theimage = file_get_contents(TEMPDIR . $filename);
				unlink(TEMPDIR . $filename);

				$boundary = "---------------------".substr(md5(rand(0,32000)),0,10);

				// Header
				$header = "POST /index.php HTTP/1.0\r\n";
				$header .= "Host: www.imageshack.us\r\n";
				$header .= "Content-type: multipart/form-data, boundary=$boundary\r\n";

				// Attach the file
				$imgdata .= "--$boundary\r\n";
				$imgdata .="Content-Disposition: form-data; name=\"fileupload\"; filename=\"" . rand(1000,9999) . ".jpg\"\r\n";
				$imgdata .= "Content-Type: image/jpeg\r\n\r\n";
				$imgdata .= "".$theimage."\r\n";
				$imgdata .="--$boundary--\r\n";

				$header .= "Content-length: " . strlen($imgdata) . "\r\n\r\n";

				$fp = fsockopen("www.imageshack.us", 80, $errno, $errstr, 30);
				fwrite($fp, $header.$imgdata);
				while (!feof($fp))
				{
					$output .= fgets($fp, 1024);
				}
				fclose($fp);

				$output = str_replace("\n", "", $output);
				preg_match("#.*value\=\"(.*)\"\/\>(.*)Direct\ link\ to\ image*#", $output, $matches);

				header("Location: " . $matches[1]);
				exit;
				/**************************************************
				</Upload to ImageShack>
				**************************************************/
			break;

			case "show":
			default:
				header("Content-type: image/jpeg");
				imagejpeg($polaroid , "", 80);
				imagedestroy($polaroid);
				exit;
			break;
		}
	}
}

if (isset($_GET['img']) && !empty($_GET['img']))
{
	$input = explode("||", base64_decode($_GET['img']));
	$_GET['bg'] = $input[0];
	$_GET['photo'] = $input[1];
	$_GET['x'] = $input[2];
	$_GET['y'] = $input[3];
	$_GET['angle'] = $input[4];
	$_GET['text'] = $input[5];
}

if ($_SERVER['REQUEST_METHOD'] == "POST" || !empty($_GET))
{
	$data = array_merge($_GET, $_POST);
	$polaroid = new Polaroid;

	if(empty($polaroid->errmsg))
	{
		if ($_POST['upload'] == "1")
		{
			$polaroid->CreatePolaroid("upload");
		}

		elseif(empty($_GET))
		{
			$url = base64_encode($data['bg'] . "||" . $polaroid->photo . "||" . $data['x'] . "||" . $data['y'] . "||" . $data['angle'] . "||" . $polaroid->text);

			if (function_exists("apache_get_modules") && in_array("mod_rewrite", apache_get_modules()))
			{
				header("Location: " . $url . "/img.jpg");
				exit;
			}
			else
			{
				header("Location: ?img=" . $url);
				exit;
			}
		}
		else
		{
			$polaroid->CreatePolaroid("show");
		}
	}
}
?>
